/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.hq;

// FIPA-OS imports
import fipaos.ont.fipa.fipaman.AgentID;
import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.util.DIAGNOSTICS;

// JAVA imports
import java.util.List;
import java.util.Iterator;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

// MY imports
import bp.meta.ont.*;
import bp.ont.OntException;





public class FilterCreator extends JFrame {

  private HQ      _owner;
  private String  _target;

  JPanel contentPane;
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  JLabel _logicL = new JLabel();
  JComboBox _logicC = new JComboBox();
  JLabel _performativeL = new JLabel();
  JComboBox _performativeC = new JComboBox();
  JLabel _senderL = new JLabel();
  JComboBox _senderC = new JComboBox();
  JLabel _receiverL = new JLabel();
  JLabel _languageL = new JLabel();
  JLabel _ontologyL = new JLabel();
  JLabel _protocolL = new JLabel();
  JButton buttonSet = new JButton();
  JButton buttonCancel = new JButton();
  JComboBox _receiverC = new JComboBox();
  JComboBox _languageC = new JComboBox();
  JComboBox _ontologyC = new JComboBox();
  JComboBox _protocolC = new JComboBox();

  public FilterCreator(HQ owner) {
    _owner= owner;

    _target= null;

    // Filter logic
    _logicC.addItem(SPYCONSTANTS.AND);
    _logicC.addItem(SPYCONSTANTS.OR);

    // FIPA performatives
    _performativeC.addItem("");
    _performativeC.addItem(FIPACONSTANTS.ACCEPT_PROPOSAL);
    _performativeC.addItem(FIPACONSTANTS.AGREE);
    _performativeC.addItem(FIPACONSTANTS.CANCEL);
    _performativeC.addItem(FIPACONSTANTS.CFP);
    _performativeC.addItem(FIPACONSTANTS.CONFIRM);
    _performativeC.addItem(FIPACONSTANTS.DISCONFIRM);
    _performativeC.addItem(FIPACONSTANTS.FAILURE);
    _performativeC.addItem(FIPACONSTANTS.INFORM);
    _performativeC.addItem(FIPACONSTANTS.INFORM_IF);
    _performativeC.addItem(FIPACONSTANTS.INFORM_REF);
    _performativeC.addItem(FIPACONSTANTS.NOT_UNDERSTOOD);
    _performativeC.addItem(FIPACONSTANTS.PROPAGATE);
    _performativeC.addItem(FIPACONSTANTS.PROPOSE);
    _performativeC.addItem(FIPACONSTANTS.PROXY);
    _performativeC.addItem(FIPACONSTANTS.QUERY_IF);
    _performativeC.addItem(FIPACONSTANTS.QUERY_REF);
    _performativeC.addItem(FIPACONSTANTS.REFUSE);
    _performativeC.addItem(FIPACONSTANTS.REJECT_PROPOSAL);
    _performativeC.addItem(FIPACONSTANTS.REQUEST);
    _performativeC.addItem(FIPACONSTANTS.REQUEST_WHEN);
    _performativeC.addItem(FIPACONSTANTS.REQUEST_WHENEVER);
    _performativeC.addItem(FIPACONSTANTS.REQUEST_WHOMEVER);
    _performativeC.addItem(FIPACONSTANTS.SUBSCRIBE);

    // FIPA-OS language constants
    _languageC.addItem("");
    _languageC.addItem(FIPACONSTANTS.SL);
    _languageC.addItem(FIPACONSTANTS.SL0);
    _languageC.addItem(FIPACONSTANTS.SL1);
    _languageC.addItem(FIPACONSTANTS.ACL);
    _languageC.addItem(FIPACONSTANTS.KIF);
    _languageC.addItem(FIPACONSTANTS.KQML);
    _languageC.addItem(FIPACONSTANTS.RDF);
    _languageC.addItem(FIPACONSTANTS.XML);
    _languageC.addItem(FIPACONSTANTS.XQL);
    _languageC.addItem(FIPACONSTANTS.OKBC);

    // FIPA protocols
    _protocolC.addItem("");
    _protocolC.addItem(FIPACONSTANTS.FIPA_REQUEST);
    _protocolC.addItem(FIPACONSTANTS.FIPA_REQUEST_WHEN);
    _protocolC.addItem(FIPACONSTANTS.FIPA_QUERY);
    _protocolC.addItem(FIPACONSTANTS.FIPA_CONTRACT_NET);
    _protocolC.addItem(FIPACONSTANTS.FIPA_ITERATED_CONTRACT_NET);
    _protocolC.addItem(FIPACONSTANTS.FIPA_AUCTION_ENGLISH);
    _protocolC.addItem(FIPACONSTANTS.FIPA_AUCTION_DUTCH);
    _protocolC.addItem(FIPACONSTANTS.FIPA_RECRUITING);
    _protocolC.addItem(FIPACONSTANTS.FIPA_BROKERING);
    _protocolC.addItem(FIPACONSTANTS.FIPA_SUBSCRIBE);


    try
    {
      jbInit();

      pack();

      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension frameSize = getSize();
      if (frameSize.height > screenSize.height)
        frameSize.height = screenSize.height;
      if (frameSize.width > screenSize.width)
        frameSize.width = screenSize.width;
      setLocation((screenSize.width - frameSize.width) / 4, (screenSize.height - frameSize.height) / 3);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void Execute(String target, List agents)
  {
    _target= target;

    setTitle("Filter for "+target);

    _senderC.removeAllItems();
    _receiverC.removeAllItems();

    // Prefill DF-registered agents
    _senderC.addItem("");
    _receiverC.addItem("");
    if(agents!=null)
    {
      Iterator i= agents.iterator();
      while(i.hasNext())
      {
        String s= (String)i.next();
        _senderC.addItem(s);
        _receiverC.addItem(s);
      }
    }

    show();
  }

  public FilterCreator() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    _logicL.setHorizontalTextPosition(SwingConstants.LEFT);
    _logicL.setText("Logic");
    contentPane= (JPanel) this.getContentPane();
    contentPane.setLayout(gridBagLayout1);
    _performativeL.setHorizontalTextPosition(SwingConstants.LEFT);
    _performativeL.setText("Performative");
    _senderL.setHorizontalTextPosition(SwingConstants.LEFT);
    _senderL.setText("Sender");
    _receiverL.setHorizontalTextPosition(SwingConstants.LEFT);
    _receiverL.setText("Receiver");
    _languageL.setHorizontalTextPosition(SwingConstants.LEFT);
    _languageL.setText("Language");
    _ontologyL.setHorizontalTextPosition(SwingConstants.LEFT);
    _ontologyL.setText("Ontology");
    _protocolL.setHorizontalTextPosition(SwingConstants.LEFT);
    _protocolL.setText("Protocol");
    Insets insets= new Insets(2, 2, 2, 2);
    buttonSet.setText("Set Filter");
    buttonSet.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonSet_actionPerformed(e);
      }
    });
    buttonCancel.setText("Cancel");
    buttonCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonCancel_actionPerformed(e);
      }
    });
    _performativeC.setEditable(true);
    _senderC.setEditable(true);
    _receiverC.setEditable(true);
    _languageC.setEditable(true);
    _ontologyC.setEditable(true);
    _protocolC.setEditable(true);
    contentPane.add(_logicL, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_logicC, new GridBagConstraints(1, 0, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_performativeL, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_performativeC, new GridBagConstraints(1, 1, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_senderL, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_senderC, new GridBagConstraints(1, 2, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_receiverL, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_receiverC, new GridBagConstraints(1, 3, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_languageL, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_languageC, new GridBagConstraints(1, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_ontologyL, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    contentPane.add(_ontologyC, new GridBagConstraints(1, 5, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    contentPane.add(_protocolL, new GridBagConstraints(0, 6, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    contentPane.add(_protocolC, new GridBagConstraints(1, 6, 2, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    contentPane.add(buttonSet, new GridBagConstraints(1, 7, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    contentPane.add(buttonCancel, new GridBagConstraints(2, 7, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
  }

  void buttonCancel_actionPerformed(ActionEvent e) {
    hide();
  }

  void buttonSet_actionPerformed(ActionEvent e) {
    MsgFilter f= new MsgFilter();

    try
    {
      f.setHQ(_owner.getAID());
      f.setLogic((String)_logicC.getSelectedItem());
      f.setPerformative((String)_performativeC.getSelectedItem());
      f.setSender((String)_senderC.getSelectedItem());
      f.setReceiver((String)_receiverC.getSelectedItem());
      f.setLanguage((String)_languageC.getSelectedItem());
      f.setOntology((String)_ontologyC.getSelectedItem());
      f.setProtocol((String)_protocolC.getSelectedItem());
      _owner.startHQTask(_target, f);
      hide();
    }
    catch(OntException oe)
    {
      DIAGNOSTICS.println("FilterCreator: OntException!?!", this, DIAGNOSTICS.LEVEL_MAX);
    }
  }
}