/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.hq;

// MY imports
import bp.meta.hq.visualizer.Visualizer;

// JAVA imports
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;


public class HQAgentGUI extends JFrame {

  HQAgent _owner;
  HQ _hq;
  Visualizer _visualizer;

  JPanel contentPane;
  JMenuBar jMenuBar1 = new JMenuBar();
  JMenu jMenuFile = new JMenu();
  JMenuItem jMenuFileExit = new JMenuItem();
  JMenu jMenuHelp = new JMenu();
  JMenuItem jMenuHelpAbout = new JMenuItem();
  JToolBar jToolBar = new JToolBar();
  ImageIcon image1;
  ImageIcon image2;
  ImageIcon image3;
  ImageIcon image4;
  ImageIcon image5;
  JLabel statusBar = new JLabel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  JSplitPane jSplitPane1 = new JSplitPane();
  JScrollPane jScrollPane1 = new JScrollPane();
  JScrollPane jScrollPane2 = null;
  JTree DFTree = null;
  JButton buttonSpy = new JButton();
  JButton buttonxSpy = new JButton();
  JButton buttonView = new JButton();
  JButton buttonxView = new JButton();
  JLabel jLabel1 = new JLabel();
  JLabel jLabel2 = new JLabel();
  JButton StatsButton = new JButton();


  /**Construct the frame*/
  public HQAgentGUI(HQAgent owner, HQ hq) {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      _owner= owner;
      _hq= hq;
      DFTree= new JTree(_owner.getDFAgentList());

      DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
      renderer.setLeafIcon(new ImageIcon(bp.meta.hq.HQAgentGUI.class.getResource("agent_bul.gif")));
      DFTree.setCellRenderer(renderer);
      DFTree.putClientProperty("JTree.lineStyle", "Angled");

      _visualizer= new Visualizer();
      jScrollPane2= _visualizer.getScrollPane();
      jbInit();

      this.setTitle(_owner.getAID().getName());

      validate();

      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension frameSize = getSize();
      if (frameSize.height > screenSize.height)
        frameSize.height = screenSize.height;
      if (frameSize.width > screenSize.width)
        frameSize.width = screenSize.width;
      setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 3);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  /**Component initialization*/
  private void jbInit() throws Exception  {
    image1 = new ImageIcon(bp.meta.hq.HQAgentGUI.class.getResource("Spy_icon.gif"));
    image2 = new ImageIcon(bp.meta.hq.HQAgentGUI.class.getResource("xSpy_icon.gif"));
    image3 = new ImageIcon(bp.meta.hq.HQAgentGUI.class.getResource("View_icon.gif"));
    image4 = new ImageIcon(bp.meta.hq.HQAgentGUI.class.getResource("xView_icon.gif"));
    image5 = new ImageIcon(bp.meta.hq.HQAgentGUI.class.getResource("Stats_icon.gif"));
    //setIconImage(Toolkit.getDefaultToolkit().createImage(HQAgentGUI.class.getResource("[Your Icon]")));
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(400, 300));
    jMenuFile.setText("File");
    jMenuFileExit.setText("Exit");
    jMenuFileExit.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuFileExit_actionPerformed(e);
      }
    });
    jMenuHelp.setText("Help");
    jMenuHelpAbout.setText("About");
    jMenuHelpAbout.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuHelpAbout_actionPerformed(e);
      }
    });
    jPanel1.setLayout(borderLayout2);
    DFTree.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        DFtreemaybePopUp(e);
      }
      public void mouseReleased(MouseEvent e) {
        DFtreemaybePopUp(e);
      }
    });
    buttonxSpy.setToolTipText("Dismiss agent");
    buttonxSpy.setIcon(image2);
    buttonxSpy.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonxSpy_actionPerformed(e);
      }
    });
    buttonView.setToolTipText("Show agent");
    buttonView.setIcon(image3);
    buttonView.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonView_actionPerformed(e);
      }
    });
    buttonSpy.setToolTipText("Recruit agent");
    buttonSpy.setIcon(image1);
    buttonSpy.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonSpy_actionPerformed(e);
      }
    });
    buttonxView.setToolTipText("Hide agent");
    buttonxView.setIcon(image4);
    buttonxView.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonxView_actionPerformed(e);
      }
    });
    statusBar.setText("  ");
    jLabel1.setText("  ");
    jLabel2.setText("  ");
    StatsButton.setToolTipText("Show agent statistics");
    StatsButton.setIcon(image5);
    StatsButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        StatsButton_actionPerformed(e);
      }
    });
    jMenuFile.add(jMenuFileExit);
    jMenuHelp.add(jMenuHelpAbout);
    jMenuBar1.add(jMenuFile);
    jMenuBar1.add(jMenuHelp);
    this.setJMenuBar(jMenuBar1);
    contentPane.add(jToolBar, BorderLayout.NORTH);
    jToolBar.add(buttonSpy, null);
    jToolBar.add(buttonxSpy, null);
    jToolBar.add(jLabel1, null);
    jToolBar.add(buttonView, null);
    jToolBar.add(buttonxView, null);
    jToolBar.add(jLabel2, null);
    jToolBar.add(StatsButton, null);
    contentPane.add(statusBar, BorderLayout.SOUTH);
    contentPane.add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(jSplitPane1, BorderLayout.CENTER);
    jSplitPane1.add(jScrollPane1, JSplitPane.LEFT);
    jScrollPane1.getViewport().add(DFTree, null);
    jSplitPane1.add(jScrollPane2, JSplitPane.RIGHT);
    jSplitPane1.setDividerLocation(160);
  }
  /**File | Exit action performed*/
  public void jMenuFileExit_actionPerformed(ActionEvent e) {
    _owner.shutdown();
  }
  /**Help | About action performed*/
  public void jMenuHelpAbout_actionPerformed(ActionEvent e) {
    AboutDialog ad= new AboutDialog();
    ad.show();
  }
  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      jMenuFileExit_actionPerformed(null);
    }
  }

  public JLabel getStatusBar()
  {
    return statusBar;
  }

  public HQListener getVisualizer()
  {
    return _visualizer;
  }

  /**
   * New agent to spy.
   */
  void buttonSpy_actionPerformed(ActionEvent e)
  {
    TreePath p= DFTree.getSelectionPath();

    if(p!=null)
      if(p.getPathCount()==3)  // Root-Platform-Agent - agent is selected
      {
        String s= (String)p.getPathComponent(2) + (String)p.getPathComponent(1);
        _hq.recruitAgent(s);
      }
  }

  /**
   * Stop spying agent.
   */
  void buttonxSpy_actionPerformed(ActionEvent e)
  {
    TreePath p= DFTree.getSelectionPath();

    if(p!=null)
      if(p.getPathCount()==3)  // Root-Platform-Agent - agent is selected
      {
        String s= (String)p.getPathComponent(2) + (String)p.getPathComponent(1);
        _hq.dismissAgent(s);
      }
  }

  /**
   * Show agent (without spying).
   */
  void buttonView_actionPerformed(ActionEvent e)
  {
    TreePath p= DFTree.getSelectionPath();

    if(p!=null)
      if(p.getPathCount()==3)  // Root-Platform-Agent - agent is selected
      {
        String s= (String)p.getPathComponent(2) + (String)p.getPathComponent(1);
        _hq.showAgent(s);
      }
  }

  /**
   * Hide agent.
   */
  void buttonxView_actionPerformed(ActionEvent e)
  {
    TreePath p= DFTree.getSelectionPath();

    if(p!=null)
      if(p.getPathCount()==3)  // Root-Platform-Agent - agent is selected
      {
        String s= (String)p.getPathComponent(2) + (String)p.getPathComponent(1);
        _hq.hideAgent(s);
      }
  }

  void DFtreemaybePopUp(MouseEvent e)
  {
    if(e.isPopupTrigger())
    {
      TreePath p = DFTree.getPathForLocation(e.getX(), e.getY());

      if(p!=null)
        if(p.getPathCount()==3)  // Root-Platform-Agent - agent is selected
        {
          String s= (String)p.getPathComponent(2) + (String)p.getPathComponent(1);
          _hq.AgentPopUp(e, s);
        }
    }
  }

  void StatsButton_actionPerformed(ActionEvent e)
  {
    TreePath p= DFTree.getSelectionPath();

    if(p!=null)
      if(p.getPathCount()==3)  // Root-Platform-Agent - agent is selected
      {
        String s= (String)p.getPathComponent(2) + (String)p.getPathComponent(1);
        _hq.showStatistics(s);
      }
  }
}