/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.hq;

// FIPA-OS imports
import fipaos.ont.fipa.ACL;
import fipaos.util.UTCContainer;

// JAVA imports
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.awt.event.*;


// MY imports


public class MessageView extends JFrame
{
  HQ   _hq;
  int  _id; // ID of currently shown message

  JPanel contentPane;
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  TitledBorder titledBorder1;
  JButton PrevButton = new JButton();
  JButton CloseButton = new JButton();
  JButton NextButton = new JButton();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  GridBagLayout gridBagLayout2 = new GridBagLayout();
  JLabel jLabel1 = new JLabel();
  JLabel jLabel2 = new JLabel();
  JLabel jLabel3 = new JLabel();
  JLabel jLabel4 = new JLabel();
  JLabel jLabel5 = new JLabel();
  JLabel jLabel6 = new JLabel();
  JLabel jLabel7 = new JLabel();
  JLabel jLabel8 = new JLabel();
  JLabel jLabel9 = new JLabel();
  JLabel jLabel10 = new JLabel();
  JLabel jLabel11 = new JLabel();
  JTextField _performativeT = new JTextField();
  JTextField _senderT = new JTextField();
  JTextField _receiversT = new JTextField();
  JTextField _replywithT = new JTextField();
  JTextField _replybyT = new JTextField();
  JTextField _inreplytoT = new JTextField();
  JTextField _languageT = new JTextField();
  JTextField _ontologyT = new JTextField();
  JTextField _protocolT = new JTextField();
  JTextField _conversationidT = new JTextField();
  Border border1;
  JScrollPane _contentSP = new JScrollPane();
  JTextArea _contentT = new JTextArea();

  public MessageView(HQ hq)
  {
    try
    {
      _hq= hq;

      jbInit();

      pack();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension frameSize = getSize();
      if (frameSize.height > screenSize.height)
        frameSize.height = screenSize.height;
      if (frameSize.width > screenSize.width)
        frameSize.width = screenSize.width;
      setLocation((screenSize.width - frameSize.width) * 3/4, (screenSize.height - frameSize.height) / 3);

    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  public void setMessage(int ID)
  {
    _id= ID;
    ACL acl= _hq.getMessage(ID);

    if(acl!=null)
    {
      _performativeT.setText(acl.getPerformative());

      _senderT.setText(acl.getSenderAID().toString());
      _senderT.setCaretPosition(0);
      _receiversT.setText(acl.getReceiverAIDs().toString());
      _receiversT.setCaretPosition(0);

      Object o= acl.getContentObject();
      if(o!=null)
        _contentT.setText(o.toString());
      else
        _contentT.setText("");
      _contentT.setCaretPosition(0);

      _replywithT.setText(acl.getReplyWith());

      UTCContainer utc= acl.getReplyByUTC();
      if(utc!=null)
        _replybyT.setText(utc.toString());
      else
        _replybyT.setText("");

      _inreplytoT.setText(acl.getInReplyTo());

      _languageT.setText(acl.getLanguage());

      _ontologyT.setText(acl.getOntology());

      _protocolT.setText(acl.getProtocol());

      _conversationidT.setText(acl.getConversationID());
    }
    else
      _performativeT.setText("null");

    show();
  }

  private void jbInit() throws Exception
  {
    contentPane= (JPanel) this.getContentPane();
    titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Message");
    border1 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Message"),BorderFactory.createEmptyBorder(10,3,3,3));
    contentPane.setLayout(borderLayout1);
    jPanel1.setBorder(border1);
    jPanel1.setLayout(gridBagLayout2);
    PrevButton.setText("<");
    PrevButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        PrevButton_actionPerformed(e);
      }
    });
    CloseButton.setText("Close");
    CloseButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        CloseButton_actionPerformed(e);
      }
    });
    NextButton.setText(">");
    NextButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        NextButton_actionPerformed(e);
      }
    });
    jPanel2.setLayout(gridBagLayout1);
    jLabel1.setText("Sender");
    jLabel2.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel2.setText("Performative");
    jLabel3.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel3.setText("Receivers");
    jLabel4.setText("Content");
    jLabel5.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel5.setText("Reply-With");
    jLabel6.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel6.setText("Reply-By");
    jLabel7.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel7.setText("In-Reply-To");
    jLabel8.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel8.setText("Language");
    jLabel9.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel9.setText("Ontology");
    jLabel10.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel10.setText("Protocol");
    jLabel11.setHorizontalTextPosition(SwingConstants.LEFT);
    jLabel11.setText("Conversation ID");
    _performativeT.setColumns(30);
    _senderT.setColumns(30);
    _receiversT.setColumns(30);
    _replywithT.setColumns(30);
    _replybyT.setColumns(30);
    _inreplytoT.setColumns(30);
    _languageT.setColumns(30);
    _ontologyT.setColumns(30);
    _protocolT.setColumns(30);
    _conversationidT.setColumns(30);
    _contentT.setColumns(30);
    _contentT.setWrapStyleWord(true);
    _contentT.setRows(5);
    _contentT.setLineWrap(true);
    contentPane.add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(jLabel2, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel1, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel3, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel4, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel5, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel6, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel7, new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel8, new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel9, new GridBagConstraints(0, 8, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel10, new GridBagConstraints(0, 9, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(jLabel11, new GridBagConstraints(0, 10, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    jPanel1.add(_performativeT, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_senderT, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_receiversT, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_replywithT, new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_replybyT, new GridBagConstraints(1, 5, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_inreplytoT, new GridBagConstraints(1, 6, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_languageT, new GridBagConstraints(1, 7, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_ontologyT, new GridBagConstraints(1, 8, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_protocolT, new GridBagConstraints(1, 9, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_conversationidT, new GridBagConstraints(1, 10, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    jPanel1.add(_contentSP, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    _contentSP.getViewport().add(_contentT, null);
    contentPane.add(jPanel2, BorderLayout.SOUTH);
    jPanel2.add(PrevButton, new GridBagConstraints(0, 0, 1, 2, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    jPanel2.add(CloseButton, new GridBagConstraints(2, 0, 1, 2, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    jPanel2.add(NextButton, new GridBagConstraints(3, 1, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
  }

  void CloseButton_actionPerformed(ActionEvent e)
  {
    _hq.deselectMessage();
    hide();
  }

  void PrevButton_actionPerformed(ActionEvent e)
  {
    if(_hq.hasPrev(_id))
      setMessage(_id-1);
  }

  void NextButton_actionPerformed(ActionEvent e)
  {
    if(_hq.hasNext(_id))
      setMessage(_id+1);
  }
}