/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.hq;

// FIPA-OS imports
import fipaos.agent.task.Task;
import fipaos.agent.task.WaitTask;
import fipaos.agent.conversation.Conversation;
import fipaos.ont.fipa.fipaman.AgentID;
import fipaos.ont.fipa.ACL;
import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.mts.Message;
import fipaos.util.DIAGNOSTICS;

// JAVA imports
import javax.swing.SwingUtilities;

// MY imports
import bp.ont.*;
import bp.meta.ont.Action;
import bp.meta.ont.MsgStats;
import bp.meta.ont.SPYCONSTANTS;


public class StatsTask extends Task
{
  public static final int TIMEOUT = 10000;

  private StatsView _sv;
  private MsgStats  _instats= null;
  private MsgStats  _outstats= null;
  private String    _error= null;

  private AgentID   _target;

  private WaitTask _wt;

  public StatsTask(StatsView sv, AgentID target)
  {
    _sv= sv;
    _target= target;
  }

  public void startTask()
  {
    Action a= new Action();

    a.setAgent(_target);
    a.setPredicate(SPYCONSTANTS.GET);
    a.setSubject(SPYCONSTANTS.STATISTICS);

    ACL acl= new ACL();

    acl.setPerformative(FIPACONSTANTS.REQUEST);
    acl.setSenderAID(_owner.getAID());
    acl.setReceiverAID(_target);

    /**
     * acl.setContentObject(a); - doesn't work if a isn't serialisable
     */
    acl.setContentObject(a);
    acl.setLanguage(SPYCONSTANTS.SPY_LANGUAGE);
    acl.setOntology(SPYCONSTANTS.SPY_ORDER);

    forward(acl);

    _wt= new WaitTask(TIMEOUT);
    newTask( _wt );

    DIAGNOSTICS.println("StatsTask started for "+_target.getName(), this, DIAGNOSTICS.LEVEL_3);
  }

  public void handleInform(Conversation conv)
  {
    ACL acl=  conv.getACL(conv.getLatestMessageIndex());

    try
    {
      Constructor c= new Constructor(acl);

      _instats= new MsgStats(c.GetStrParam(SPYCONSTANTS.INCOMMING));
      _outstats= new MsgStats(c.GetStrParam(SPYCONSTANTS.OUTGOING));
    }
    catch(Exception e)
    {
      _instats= null;
      _outstats= null;
      _error= "Invalid statistics";
    }
    done();
  }

  public void handleNotUnderstood(Conversation c)
  {
    _error= "Statistics unsupported";
    done();  //Some error ocured
  }

  public void handleFailure(Conversation c)
  {
    _error= "Failure received";
    done();
  }

  public void doneWaitTask(Task t)
  {
    _wt= null;
    _error= "Operation timed out";
    done();
  }

  protected void done()
  {
    // Stop the wait task to awoit _tm warning
    if(_wt!=null)
      _tm.removeTask(_wt);

//      if( (_instats==null) || (_outstats==null))
//        _sv.error(_error);
//      else
//        _sv.deliver(_instats, _outstats);
    SwingUtilities.invokeLater(new SwingNotifier());

    if(_error==null)
      DIAGNOSTICS.println("StatsTask finished succesfully for "+_target.getName(), this, DIAGNOSTICS.LEVEL_3);
    else
      DIAGNOSTICS.println("StatsTask failed for "+_target.getName(), this, DIAGNOSTICS.LEVEL_3);

    super.done();
  }

  private class SwingNotifier implements Runnable
  {
    public void run()
    {
      if( (_instats==null) || (_outstats==null))
        _sv.error(_error);
      else
        _sv.deliver(_instats, _outstats);
    }
  }
}