/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.hq;

// FIPA-OS imports
import fipaos.ont.fipa.fipaman.AgentID;

// MY imports
import bp.meta.ont.MsgStats;

// JAVA imports
import java.util.*;
import javax.swing.table.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.awt.event.*;



public class StatsView extends JFrame
{
  HQAgent  _owner= null;
  AgentID _target= null;
  ResultTable _results= new ResultTable();

  boolean closed= false;

  JPanel contentPane;
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel CardPanel = new JPanel();
  JPanel jPanel2 = new JPanel();
  CardLayout cardLayout = new CardLayout();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  JButton ClearButton = new JButton();
  JButton CloseButton = new JButton();
  TitledBorder titledBorder1;
  Border border1;
  JPanel StatsPanel = new JPanel();
  JPanel ErrorPanel = new JPanel();
  JLabel ErrorLabel = new JLabel();
  BorderLayout borderLayout2 = new BorderLayout();
  JScrollPane jScrollPane1 = new JScrollPane();
  BorderLayout borderLayout3 = new BorderLayout();
  JTable jTable1 = new JTable(_results);
  JButton RefreshButton = new JButton();

  public StatsView(HQAgent owner, AgentID target)
  {
    super();
    _owner= owner;
    _target= target;
    doInit();
    jTable1.getColumnModel().getColumn(0).setPreferredWidth(90);
    jTable1.getColumnModel().getColumn(1).setPreferredWidth(20);
    jTable1.getColumnModel().getColumn(2).setPreferredWidth(20);
  }

  public StatsView()
  {
    doInit();
  }

  private void doInit()
  {
    try
    {
      if(_target!=null)
        setTitle(_target.getName());
      jbInit();
      cardLayout.show(CardPanel, "ErrorPanel");
      _owner.newTask(new StatsTask(this, _target));

      pack();

      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension frameSize = getSize();
      if (frameSize.height > screenSize.height)
        frameSize.height = screenSize.height;
      if (frameSize.width > screenSize.width)
        frameSize.width = screenSize.width;
      setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  public void error(String s)
  {
    if(!closed)
    {
      ErrorLabel.setText(s);
      cardLayout.show(CardPanel, "ErrorPanel");
    }
  }

  public void deliver(MsgStats _in, MsgStats _out)
  {
    if(!closed)
    {
      _results.clear();

      SortedSet s= new TreeSet(_in.getPerformativesSet());
      s.addAll(_out.getPerformativesSet());

      Iterator i= s.iterator();
      while(i.hasNext())
      {
        String p= (String)i.next();
        int   in= _in.get(p);
        int  out= _out.get(p);
        _results.insertRow(p, in, out);
      }

      _results.insertRow("TOTAL", _in.getTotal(), _out.getTotal());

      // fill in the table
      if(_target!=null)
        ClearButton.setEnabled(true);
      cardLayout.show(CardPanel, "StatsPanel");
    }
  }

  private class ResultTable extends AbstractTableModel {
    final String[] _columnNames = { "Performative", "in", "out" };
    Vector _data= new Vector();

    private class TabRow extends Object
    {
      public final String _p;
      public final int _in;
      public final int _out;

      public TabRow(String p, int i, int o)
        { _p= p; _in= i; _out= o; }
    }

    public int getColumnCount() {
        return _columnNames.length;
    }

    public int getRowCount() {
        return _data.size();
    }

    public String getColumnName(int col) {
      return _columnNames[col];
    }

    public Object getValueAt(int row, int col) {
      if(col==0)
        return ((TabRow)_data.get(row))._p;
      else if(col==1)
        return new Integer(((TabRow)_data.get(row))._in);
      else
        return new Integer(((TabRow)_data.get(row))._out);
    }

    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }

    public void insertRow(String p, int i, int o)
    {
      int r= _data.size();
      _data.add(r, new TabRow(p, i, o));
      fireTableRowsInserted(r, r);
    }

    public void clear()
    {
      int r= _data.size();
      if(r>0)
      {
        _data.clear();
        fireTableRowsDeleted(0, r-1);
      }
    }
  }

  private void jbInit() throws Exception
  {
    contentPane= (JPanel) this.getContentPane();
    titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Statistics");
    border1 = BorderFactory.createCompoundBorder(titledBorder1,BorderFactory.createEmptyBorder(5,5,5,5));
    contentPane.setLayout(borderLayout1);
    CardPanel.setLayout(cardLayout);
    jPanel2.setLayout(gridBagLayout1);
    ClearButton.setEnabled(false);
    ClearButton.setText("Clear");
    ClearButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ClearButton_actionPerformed(e);
      }
    });
    CloseButton.setText("Close");
    CloseButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        CloseButton_actionPerformed(e);
      }
    });
    CardPanel.setBorder(border1);
    ErrorLabel.setHorizontalAlignment(SwingConstants.CENTER);
    ErrorLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    ErrorLabel.setText("Requesting Statistics...");
    ErrorPanel.setLayout(borderLayout2);
    StatsPanel.setLayout(borderLayout3);
    this.addWindowListener(new java.awt.event.WindowAdapter()
    {
      public void windowClosed(WindowEvent e)
      {
        this_windowClosed(e);
      }
    });
    jScrollPane1.setPreferredSize(new Dimension(150, 200));
    RefreshButton.setText("Refresh");
    RefreshButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        RefreshButton_actionPerformed(e);
      }
    });
    contentPane.add(CardPanel, BorderLayout.CENTER);
    CardPanel.add(ErrorPanel, "ErrorPanel");
    ErrorPanel.add(ErrorLabel, BorderLayout.CENTER);
    CardPanel.add(StatsPanel, "StatsPanel");
    StatsPanel.add(jScrollPane1, BorderLayout.CENTER);
    jScrollPane1.getViewport().add(jTable1, null);
    contentPane.add(jPanel2, BorderLayout.SOUTH);
    jPanel2.add(ClearButton, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 5, 5, 5), 0, 0));
    jPanel2.add(CloseButton, new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 5, 5, 5), 0, 0));
    jPanel2.add(RefreshButton, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 5, 5, 5), 0, 0));
  }

  void CloseButton_actionPerformed(ActionEvent e)
  {
    closed= true;
    hide();
//    dispose();
  }

  void this_windowClosed(WindowEvent e)
  {
    closed= true;
    hide();
//    dispose();
  }

  void ClearButton_actionPerformed(ActionEvent e)
  {
    _owner.newTask(new ClearStatsTask(this, _target));
    ErrorLabel.setText("Sending CLEAR request...");
    cardLayout.show(CardPanel, "ErrorPanel");
  }

  void RefreshButton_actionPerformed(ActionEvent e)
  {
    _owner.newTask(new StatsTask(this, _target));
    ErrorLabel.setText("Requesting statistics...");
    cardLayout.show(CardPanel, "ErrorPanel");
  }
}