/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.hq.visualizer;

// JAVA imports
import java.util.Vector;
import java.awt.*;
import javax.swing.*;


public class AgentVisualizer extends JComponent {

  private static final String OTHERS = "Others";

  private Vector     _agents;

  public AgentVisualizer(Vector agents)
  {
    _agents= agents;
    setPreferredSize(new Dimension(VCONST.INIT_W, VCONST.HEADER_HEIGHT));
  }

  public void setAgentCount(int cnt)
  {
    setPreferredSize(new Dimension((cnt+1)*VCONST.BOX_X+VCONST.BOX_OFS, VCONST.HEADER_HEIGHT));
    revalidate();
  }

  public void paintComponent(Graphics g)
  {
    Graphics2D g2= (Graphics2D)g;

    Rectangle drawArea = g2.getClipBounds();

    // Fill clipping area with white.
    g2.setColor(new Color(255, 255, 255));
    g2.fillRect(drawArea.x, drawArea.y, drawArea.width, drawArea.height);


    FontMetrics metrics = g2.getFontMetrics();

    double firstd= (drawArea.getX()-VCONST.BOX_OFS)/VCONST.BOX_X;
    int first;
    if(firstd<0)
      first= 0;
    else
      first= (int)firstd;

    int last= first + (int)(drawArea.getWidth()/VCONST.BOX_X)+1;
    if(last>_agents.size())
      last= _agents.size();

    g2.setFont(getFont().deriveFont(Font.BOLD));

    if(first<1)
    {
      g2.setColor(Color.darkGray);
      g2.fill(new Rectangle(VCONST.BOX_OFS, VCONST.BOX_Y, VCONST.BOX_WIDTH, VCONST.BOX_HEIGHT));
      g2.setColor(Color.yellow);

      int w = metrics.stringWidth( OTHERS );
      int h = metrics.getHeight();
      g2.drawString( OTHERS, VCONST.BOX_OFS+VCONST.BOX_WIDTH/2-w/2,
                             VCONST.BOX_Y+ VCONST.BOX_HEIGHT/2+h/2 );
      first= 0;
    }
    else
      first--;

    g2.setStroke(new BasicStroke(2));

    for(int i= first; i<last; i++)
    {
      VisAgent a= (VisAgent)_agents.get(i);
      g2.setColor(a._back);
      g2.fill(a.getImage());

      if(a._spy)  // Draw a frame for spyed agents
      {
        g2.setColor(Color.black);
        g2.draw(a.getImage());
      }

      g2.setColor(a._front);

      int width = metrics.stringWidth( a._name );
      int height = metrics.getHeight();
      g2.drawString( a._name, (i+1)*VCONST.BOX_X+VCONST.BOX_OFS+VCONST.BOX_WIDTH/2-width/2,
                             VCONST.BOX_Y+ VCONST.BOX_HEIGHT/2+height/2 );
    }
  }

}