/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */


package bp.meta.hq.visualizer;

// FIPA-OS imports
import fipaos.util.DIAGNOSTICS;

// JAVA imports
import java.util.Vector;
import javax.swing.JPanel;
import java.awt.*;



public class MessageVisualizer extends JPanel {

//  private static final int HEIGHT     = 50;
//  private static final int BOX_HEIGHT = 30;
//  private static final int BOX_WIDTH  = 75;
//  private static final int BOX_W2     = (new Double(BOX_WIDTH/2)).intValue();
//  private static final int BOX_GAP    = 20;
//  private static final int BOX_Y      = HEIGHT-BOX_HEIGHT;
//  private static final int BOX_X      = BOX_WIDTH+BOX_GAP;

  private int    _agcnt= 1;
  private int    _msgcnt= 0;
  private int    _selected= -1;
  private Vector _msgs;

  public MessageVisualizer(Vector msgs)
  {
    _msgs= msgs;
//    setPreferredSize(new Dimension(600, 500));
  }

  public void setAgentCount(int cnt)
  {
    int h= cnt*VCONST.ROW_H+VCONST.ROW_OFS;
    if(h<100)
      h= 100;
    setPreferredSize(new Dimension((cnt+1)*VCONST.BOX_X+VCONST.BOX_OFS, h));
    _agcnt= cnt+1;
    revalidate();
  }

  public void setMsgCount(int cnt)
  {
    _msgcnt= cnt;
    int h= cnt*VCONST.ROW_H+VCONST.ROW_OFS;
    if(h<100)
      h= 100;
    setPreferredSize(new Dimension(_agcnt*VCONST.BOX_X+VCONST.BOX_OFS, h));
    revalidate();
  }

  public void highlightMessage(int ID)
  {
    _selected= ID;
    repaint();
  }

  public void paintComponent(Graphics g1)
  {
    Graphics2D g2= (Graphics2D)g1;
    Rectangle drawArea = g2.getClipBounds();

    // Fill clipping area with white.
    g2.setColor(new Color(255, 255, 255));
    g2.fillRect(drawArea.x, drawArea.y, drawArea.width, drawArea.height);

    g2.setColor(new Color(0, 0, 0));
    for(int i= 0; i<_agcnt; i++)
    {
      g2.drawLine( i*VCONST.COL_X+VCONST.COL_OFS, drawArea.y,
                  i*VCONST.COL_X+VCONST.COL_OFS,  drawArea.y+drawArea.height);
    }

    if(_msgs.size()!=0)
    {
      int first= (int) ((drawArea.y-VCONST.ROW_OFS)/VCONST.ROW_H)-2;
      if(first<0)
        first= 0;

      int last= first + (int)(drawArea.getHeight()/VCONST.ROW_H)+5;

      for(int i= 0; i<_msgs.size(); i++)
      {
        VisMessage vm= (VisMessage)_msgs.get(i);
        if(vm.getRow()>=first)
        {
          if(vm._ID==_selected)
            g2.setColor(Color.red);
          else
            g2.setColor(Color.orange);
          g2.fill(vm.getImage());
          if(vm.getNextRow()>last)
            break;
        }
      }
    }
  }


  }