/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */


package bp.meta.hq.visualizer;

// JAVA imports
import java.util.List;
import java.util.Iterator;
import java.awt.Color;
import java.awt.Polygon;
import java.awt.Point;
import java.awt.geom.*;

public class VisAgent {

  public final String _name;
  public boolean _spy;

  public final String _fullname;
  private Area   _image;

  public final Color _front;
  public final Color _back;

  public VisAgent( int pos, String name, boolean spy, Color front, Color back)
  {
    _fullname= name;
    _name= name.substring(0,name.indexOf('@'));
    _spy= spy;
    _front= front;
    _back= back;
    createImage(pos);

  }

  public void createImage(int pos)
  {
    _image= new Area(new Rectangle2D.Double(pos*VCONST.BOX_X+VCONST.BOX_OFS, VCONST.BOX_Y, VCONST.BOX_WIDTH, VCONST.BOX_HEIGHT));
  }

  public Area getImage() { return _image; }

  public boolean equals(Object o)
  {
    return _fullname.equals(o);
  }

  public static class StringComparator
  {
    private String _s;

    public StringComparator(String s) { _s= s; }

    public boolean equals(Object o)
    {
      if(o instanceof VisAgent)
        return _s.equalsIgnoreCase(((VisAgent)o)._fullname);
      else
        return _s.equals(o);
    }
  }

  public static class PointComparator
  {
    private Point _p;

    public PointComparator(Point p)
    {
      _p= p;
    }

    public boolean equals(Object o)
    {
      if(o instanceof VisAgent)
        return ((VisAgent)o)._image.contains(_p);
      else
        return super.equals(o);
    }
  }
}