/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        Spy Headquaters Agent
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.hq.visualizer;

// FIPA-OS imports
import fipaos.ont.fipa.ACL;
import fipaos.ont.fipa.fipaman.AgentID;

// JAVA imports
import java.util.Vector;
import java.util.List;
import java.util.Iterator;
import java.awt.Point;
import java.awt.geom.*;
import java.awt.Polygon;


public class VisMessage
{
  public final int _ID;

  private int      _row;
  private Vector   _agents;             // Refference to the agent list

  private String   _performative;
  private String   _sender;
  private String[] _receivers;

  private Area     _image;

  public VisMessage(int ID, int row, Vector agents, ACL msg)
  {
    _ID= ID;
    _row= row;
    _agents= agents;

    _performative= msg.getPerformative();
    _sender= msg.getSenderAID().getName();

    List l= msg.getReceiverAIDs();   // Fill in the receivers
    _receivers= new String[l.size()];
    Iterator i= l.iterator();
    int c= 0;
    while(i.hasNext())
    {
      _receivers[c]= ((AgentID)i.next()).getName();
      c++;
    }

    createImage();
  }

  public void createImage()
  {
    int   start= _agents.indexOf(new VisAgent.StringComparator(_sender));
    int[] ends= new int[_receivers.length];

    start++;  // -1 -> 0 (First one is "NOT_FOUND"

    _image= new Area(new Rectangle2D.Double( start*VCONST.COL_X+VCONST.COL_OFS-VCONST.BAR_W2, _row*VCONST.ROW_H+VCONST.ROW_OFS-VCONST.BAR_OFS,
                                           VCONST.BAR_W, (_receivers.length-1)*VCONST.ROW_H+2*VCONST.BAR_OFS));

    for(int i= 0; i<_receivers.length; i++)
      _image.add(createArrow(_row+i, start, _agents.indexOf(new VisAgent.StringComparator(_receivers[i]))+1 ));
  }

  public int getRow()             { return _row;}
  public int getNextRow()         { return _row+_receivers.length; }
  public Area getImage()          { return _image; }
  public String getPerformative() { return _performative; }


  private Area createArrow(int row, int start, int end)
  {
    Area     a;
    Polygon  tip;

    int startx= start*VCONST.COL_X+VCONST.COL_OFS;
    int endx= end*VCONST.COL_X+VCONST.COL_OFS;
    int y= row*VCONST.ROW_H+VCONST.ROW_OFS;

//    a= new Area(new Line2D.Double(startx, y, endx, y));


    if(start<end)    // "->"
    {
      tip= new Polygon(new int[] {endx-VCONST.TIP_LEN, endx, endx-VCONST.TIP_LEN},
                       new int[] {y-VCONST.TIP_W2, y, y+VCONST.TIP_W2}, 3);
      a= new Area(new Rectangle2D.Double(startx, y-VCONST.LINE_W2, (endx-startx)-VCONST.TIP_LEN, VCONST.LINE_W));
    }
    else             // "<-"
    {
      tip= new Polygon(new int[] {endx+VCONST.TIP_LEN, endx, endx+VCONST.TIP_LEN},
                       new int[] {y-VCONST.TIP_W2, y, y+VCONST.TIP_W2}, 3);
      a= new Area(new Rectangle2D.Double(endx+VCONST.TIP_LEN, y-VCONST.LINE_W2, (startx-endx)-VCONST.TIP_LEN, VCONST.LINE_W));
    }

    a.add(new Area(tip));
    return a;
  }

  public static class PointComparator
  {
    private Point _p;

    public PointComparator(Point p)
    {
      _p= p;
    }

    public boolean equals(Object o)
    {
      if(o instanceof VisMessage)
        return ((VisMessage)o)._image.contains(_p);
      else
        return super.equals(o);
    }
  }
}