/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        MessageSpy Ontology
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.ont;

// FIPA OS imports
import fipaos.ont.fipa.fipaman.AgentID;
import fipaos.ont.BASETOKENS;
import fipaos.parser.sl.SLParser;
import fipaos.parser.sl.ParseException;
import fipaos.agent.conversation.content.Content;

// JAVA imports
import java.io.Serializable;

// My imports
import bp.ont.OntException;

public class Action implements Serializable {

  private AgentID _agent;
  private String  _predicate;
  private String  _subject;

  public Action() {
    _agent = new AgentID();
    _predicate= null;
    _subject= null;
  }

  public Action(String content) throws OntException
  {
    try
    {
      Content c= SLParser.parseActionExpression(content);

      _agent= new AgentID((String)((Content)c.getAttribute("agent")).getAttribute("formattedString"));
      c= (Content)c.getAttribute("term");
      c= (Content)c.getAttribute("functionalTerm");
      _predicate= (String)c.getAttribute("functionalOperator");
      c= (Content)c.getAttribute("term");
      _subject= (String)c.getAttribute("formattedString");
    }
    catch(Throwable e)
    {
      throw new OntException("Wrong Action: \""+content+"\"\n"+e.getMessage());
    }
  }

  public AgentID getAgent() { return _agent; }
  public String  getAgentName() { return _agent.getName(); }
  public String  getPredicate() { return _predicate; }
  public String  getSubject() { return _subject; }

  public void setAgent(AgentID agent) { _agent= agent; }
  public void setPredicate(String predicate) { _predicate= predicate; }
  public void setSubject(String subject) { _subject= subject; }

  public boolean comparePredicate(String s)
    { return _predicate.equalsIgnoreCase(s); }

  public boolean compareSubject(String s)
    { return _subject.equalsIgnoreCase(s); }

  public String toString()
  {
    return "("+BASETOKENS.ACTION+" "+_agent+" ("+_predicate+" "+_subject+"))";
  }
}