/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        MessageSpy Ontology
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.ont;

// FIPA OS imports
import fipaos.parser.sl.ParseException;

// Java imports
import java.util.*;

// My imports
import bp.ont.*;
import bp.meta.ont.Action;

public class MsgStats {

  private int _total= 0;
  private TreeMap _sorted= null;

  public MsgStats()
  {
    _sorted= new TreeMap();
  }

  public MsgStats(String stats) throws OntException
  {
    _sorted= new TreeMap();
    _total= 0;
    try
    {
      Constructor c= new Constructor(stats);
      if(c.CompareOp(SPYCONSTANTS.STATISTICS))
      {
        Iterator i= c.GetParamSet().iterator();
        while(i.hasNext())
        {
          String s= (String)i.next();  // substring to remove initial :
          if(s.equalsIgnoreCase(SPYCONSTANTS.TOTAL))
            _total= c.GetIntParam(s);
          else
            _sorted.put(s.substring(1), new Integer(c.GetIntParam(s)));
        }
      }
      else
        throw new OntException("Not a valid statistics: "+stats);
    }
    catch(ParseException e)
    {
      throw new OntException("Not a valid statictics: "+stats);
    }
  }

  public void clear()
  {
    _total= 0;
    _sorted.clear();
  }

  public void insert(String performative)
  {
    ++_total;
    Integer i= (Integer)_sorted.get(performative);
    if(i==null)
      i= new Integer(0);

    _sorted.put(performative, new Integer(i.intValue()+1));
  }

  public int get(String performative)
  {
    Object o= _sorted.get(performative);
    if(o==null)
      return 0;
    else
      return ((Integer)o).intValue();
  }

  public int getTotal()
  {
    return _total;
  }

  public Set getPerformativesSet()
  {
    if(_sorted!=null)
      return _sorted.keySet();
    else
      return null;
  }

  public String toString()
  {
    String s= new String();

    s= SPYCONSTANTS.TOTAL+" "+_total;

    Iterator i= _sorted.keySet().iterator();
    while(i.hasNext())
    {
      String perf= (String)i.next();
      s= s+ " :"+ perf+ " " +_sorted.get(perf);
    }

    return "("+SPYCONSTANTS.STATISTICS+" "+s+")";
  }
}