/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        MessageSpy util
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.meta.util;

// JAVA imports
import java.util.StringTokenizer;
import java.util.List;
import java.util.ArrayList;

public class StringPattern {

  private static final String DELIMITERS= "*?";
  private static final int INF= 0xffff;

  private List _tokens= null;
  private String _pattern;
  private boolean _complete;

  public StringPattern(String Pattern)
  {
    if(Pattern==null)
      _pattern= "";
    else
      _pattern= Pattern.toLowerCase();

    if(_pattern!="")
    {
      _complete= (_pattern.indexOf('*')==-1) && (_pattern.indexOf('?')==-1);

      if(!_complete)
      {
        _tokens= new ArrayList();

        StringTokenizer t= new StringTokenizer(Pattern, DELIMITERS, true);

        int min= 0;
        int max= 0;
        while(t.hasMoreTokens())
        {
          String s= t.nextToken();
          if(s.equals("*"))
            max= INF;
          else if(s.equals("?"))
            min++;
          else
          {
            if(max<min)
              max= min;
            _tokens.add(new Token(s, min, max));
            min= 0;
            max= 0;
          }
        }
        if( (min!=0) || (max!=0) )   // There was * or ? at the end.
          _tokens.add(new Token( "", min, max));
      }
    }
  }

  public boolean Match(String s, boolean Default)
  {
    if(_pattern=="")
      return Default;

    if(s==null)
      s= "";

    if(_complete)
      return _pattern.equalsIgnoreCase(s);
    else
    {
      int[] poss= new int[_tokens.size()];
      int tpos = 0;
      int spos = 0;

      s= s.toLowerCase();

      while(tpos<_tokens.size())
      {
        Token t= (Token)_tokens.get(tpos);

        int i= s.indexOf(t.s(), spos+t.min()); // Find the token

        if(i==-1)
          return false;
        if(i>(spos+t.max()))  // Token too far
        {
          if(tpos>0)
          {
            --tpos;
            spos= ++poss[tpos];
          }
          else
            return false; // Token not found- doesn't match
        }
        else
        {
          poss[tpos]= i;
          spos= i+t.len();
          tpos++;
        }
      }
      return true;
    }
  }

  public boolean Empty()
  {
    return _pattern=="";
  }

  public String toString()
  {
    return _pattern;
  }

  private class Token {
    private String _s;
    private int _minofs;
    private int _maxofs;

    public Token(String s, int min, int max)
    {
      _s= s;
      _minofs= min;
      _maxofs= max;
    }

    public int len() { return _s.length(); }
    public String s() { return _s; }
    public int min() { return _minofs; }
    public int max() { return _maxofs; }
  }
}