/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        MessageSpy Ontology
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.ont;

// Java imports
import java.util.*;

// Import the fipa classes
import fipaos.ont.fipa.*;
import fipaos.parser.sl.SLParser;
import fipaos.parser.sl.ParseException;

import fipaos.agent.conversation.content.*;

// Finally import the diagnostics class for output
import fipaos.util.DIAGNOSTICS;

/**
 * Title:        Ping Pong Agent Implementation
 * Description:  My first MAS
 * Copyright:    Copyright (c) 2001
 * Company:      none
 * @author MiDo
 * @version 1.0
 */

public class Constructor {

  private String    _op;
  private Hashtable _params;

  public Constructor(String Op, Hashtable Params)
  {
    _op= Op;
    _params= Params;
  }

  /**
   * Initialise from ACL message with Content in a form of SL FunctionalTerm
   * In a form of (operator :param1key param1value :param2key param2value ...)
   */
  public Constructor(ACL msg) throws ParseException
  {
    Object c;
    c= msg.getContentObject();
    fillIn((String)c);
  }

  /**
   * Initialize by String representing a FunctionalTerm in a form
   * (operator :param1key param1value :param2key param2value ...)
   */
  public Constructor(String s) throws ParseException
  {
    fillIn(s);
  }

  private void fillIn(String ContentString) throws ParseException
  {
    _op= null;
    _params= new Hashtable();

    try
    {
      // Parse the input term
      Content cnt= (Content)SLParser.parseTerm(ContentString).getAttribute("functionalTerm");
      // Get the operator keyword
      _op= (String)cnt.getAttribute("functionalOperator");

      // Set of parameters
      Object par= cnt.getAttribute("parameter");
      if(par instanceof LinkedList)
      {  // More parameters?
        Iterator i= ((LinkedList)par).iterator();
        while(i.hasNext())
        {
          Content p= (Content)i.next();
          String parname= (String)p.getAttribute("parameterName");
          String parval= (String)((Content)p.getAttribute("parameterValue")).getAttribute("formattedString");
          _params.put(parname,parval);
        }
      }
      else if(par instanceof Content)
      {  // just one parameter.
          Content p= (Content)par;
          String parname= (String)p.getAttribute("parameterName");
          String parval= (String)((Content)p.getAttribute("parameterValue")).getAttribute("formattedString");
          _params.put(parname,parval);
      }
    }
    catch (fipaos.parser.sl.ParseException e)
    {
      DIAGNOSTICS.println( "PA: Parser Error.  Content:"+e.getMessage(), DIAGNOSTICS.LEVEL_4 );
      throw e;
    }
  }

  public String GetOp()
  {
    return _op.toLowerCase();
  }

  /**
   * Compare the Message operator to value o
   */
  public boolean CompareOp(String o)
  {
    return _op.equalsIgnoreCase(o);
  }

  public int GetIntParam(String key)
  {
    return java.lang.Integer.decode((String)_params.get(key)).intValue();
  }

  public String GetStrParam(String key)
  {
    return (String)_params.get(key);
  }

  public Set GetParamSet()
  {
    if(_params!=null)
      return _params.keySet();
    else
      return null;
  }

  public int size()
  {
    return _params.size();
  }
}