/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        PingPong agent for MessageSpy testing
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.pingpong;

//import com.borland.jbcl.layout.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import com.klg.jclass.field.*;
import java.awt.event.*;
import java.util.*;

// FIPA-OS imports
import fipaos.ont.fipa.fipaman.*;  //DFAgentDescription & AgentID
import fipaos.util.DIAGNOSTICS;

/**
 * Title:        Ping Pong Agent Implementation
 * Description:  My first MAS
 * Copyright:    Copyright (c) 2001
 * Company:      none
 * @author MiDo
 * @version 1.0
 */

public class PingPongGUI extends JFrame {

  String _name= null;
  PingPongAgent _owner= null;
  ResultTable _results = new ResultTable();


  JPanel contentPane;
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  JLabel jLabel1 = new JLabel();
  JTextField countText = new JTextField();
  JLabel jLabel2 = new JLabel();
  JComboBox opponentCombo = new JComboBox();
  JButton jButton1 = new JButton();
  Border border1;
  JScrollPane jScrollPane1 = new JScrollPane();
  JTable jTable1 = new JTable(_results);
  JButton jButton2 = new JButton();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  GridBagLayout gridBagLayout2 = new GridBagLayout();
  BorderLayout  borderLayout2 = new BorderLayout();
  Border border2;
  Border border3;

  public PingPongGUI(PingPongAgent Owner) {
    try {
      _name= Owner.getAID().getName();
      _owner= Owner;
      jbInit();
      pack();

      _owner.StartDFSearch();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception {
    contentPane = (JPanel) this.getContentPane();
    border1 = BorderFactory.createLineBorder(Color.gray,2);
    border2 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
    border3 = BorderFactory.createEmptyBorder(1,1,1,1);
    contentPane.setLayout(borderLayout1);
    jPanel1.setLayout(borderLayout2);
    jPanel2.setLayout(gridBagLayout1);
    this.setTitle(_name);

    jLabel1.setToolTipText("Number of hame hops");
    jLabel1.setText("Count:");
    countText.setMinimumSize(new Dimension(25, 21));
    countText.setToolTipText("Number of game hops");
    countText.setText("5");
    countText.setColumns(3);
    jLabel2.setToolTipText("Agent to play with");
    jLabel2.setText("Opponent:");
    jButton1.setToolTipText("Start game");
    jButton1.setText("Play!");
    jButton1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButton1_actionPerformed(e);
      }
    });
    opponentCombo.setMaximumSize(new Dimension(140, 21));
    opponentCombo.setMinimumSize(new Dimension(140, 21));
    opponentCombo.setToolTipText("Agent to play with");
    opponentCombo.setEditable(true);
    jButton2.setToolTipText("Refresh opponent list by DF search");
    jButton2.setText("Refresh");
    jButton2.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButton2_actionPerformed(e);
      }
    });
    jPanel2.setBorder(border2);
    jScrollPane1.setPreferredSize(new Dimension(450, 160));
    jPanel1.setBorder(border3);
    jPanel2.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 10, 5, 0), 0, 0));
    jPanel2.add(countText, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 10), 0, 0));
    jPanel2.add(jLabel2, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
    jPanel2.add(opponentCombo, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 10), 0, 0));
    jPanel2.add(jButton2, new GridBagConstraints(4, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
//    new GridBagConstraints(0, 1, 5, 1, 1.0, 0.0
//            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 0, 0, 0), 0, 0));
    jPanel1.add(jPanel2, BorderLayout.NORTH);
    jPanel1.add(jButton1, BorderLayout.SOUTH);
//    new GridBagConstraints(4, 0, 1, 1, 1.0, 0.0
//            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    contentPane.add(jPanel1, BorderLayout.NORTH);
    contentPane.add(jScrollPane1, BorderLayout.CENTER);
    jScrollPane1.getViewport().add(jTable1, null);
  }

  public void SearchResults(DFAgentDescription descs[])
  {
    // Check we've got some results - if not just return
    if( descs == null || descs.length <= 0 )
    {
      return;
    }

    opponentCombo.removeAllItems();

    for( int i = 0; i < descs.length; i++ )
      opponentCombo.addItem(descs[i].getAgentID().getName());
  }

  private class ResultTable extends AbstractTableModel {
    final String[] _columnNames = { "Result", "Opponent" };
    private java.util.List _data = Collections.synchronizedList( new LinkedList() );

    private class TabRow extends Object
    {
      public String _result;
      public String _opponent;

      public TabRow(String Result, String Opponent)
      {
        _result= Result;
        _opponent= Opponent;
      }
    }

    public int getColumnCount() {
        return _columnNames.length;
    }

    public int getRowCount() {
        return _data.size();
    }

    public String getColumnName(int col) {
      return _columnNames[col];
    }

    public Object getValueAt(int row, int col) {
      if(col==0)
        return ((TabRow)_data.get(row))._result;
      else
        return ((TabRow)_data.get(row))._opponent;
    }

    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }

    public void insertResult(String Result, String Opponent)
    {
      int r= _data.size();
      _data.add(r, new TabRow(Result, Opponent));
      fireTableRowsInserted(r, r);
    }
  }

  public void insertResult(String Result, String Opponent)
  {
    _results.insertResult(Result,Opponent);
  }

  void jButton1_actionPerformed(ActionEvent e) {
    try
    {
      AgentID id= new AgentID();
      id.setName((String)opponentCombo.getSelectedItem());
      _owner.StartPingPong(null, id, Integer.parseInt(countText.getText()));
    }
    catch(Throwable t)
    {
      DIAGNOSTICS.println( "GO! Error:"+t, DIAGNOSTICS.LEVEL_4 );
    }
  }

  void jButton2_actionPerformed(ActionEvent e) {
    try
    {
      _owner.StartDFSearch();
    }
    catch(Throwable t)
    {
    }
  }

}