/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        PingPong agent for MessageSpy testing
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.pingpong;

// Agent types
import fipaos.agent.task.*;
import fipaos.agent.conversation.*;

// Ontology types
import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;

// Finally import the diagnostics class for output
import fipaos.util.DIAGNOSTICS;

// My imports
import bp.ont.Constructor;

/**
 * PingPongTask is the parent class for Pong- and Ping- Tasks.
 */
public class PingPongTask extends Task {

  private final int TIMEOUT= 5000;

  protected String _playertype= null;

  protected AgentID _opponent= null;
  protected String _result= "(failed)";
  protected String _gamestate= "Failed";
  private TimeoutTask _ttask= null;

  public PingPongTask(String PlayerType)
  {
    _playertype= PlayerType;
//    _waitt= new WaitTask(TIMEOUT);
  }

  protected void startTask()
  {
//    newTask(_waitt);
  }

  public void handleRequest( Conversation conv )
  {
    try
    {
      ACL acl = conv.getACL(conv.getLatestMessageIndex());

      Constructor msg= new Constructor(acl);

      if(msg.CompareOp(Neg(_playertype)))
      {
        int n= msg.GetIntParam(":count");

        if(n>0)   // Send response, keep playing
        {
          n--;
          ACL resp = conv.getFilledInACL();
          resp.setSenderAID( _owner.getAID() );
          resp.setReceiverAID( _opponent );
          resp.setPerformative( FIPACONSTANTS.REQUEST );
          resp.setContentObject( "("+_playertype+" :count " + n + ")" );
          resp.setOntology( "ping-pong" );
          forward( resp );

          // Indicate what is going on
          DIAGNOSTICS.println( _playertype+": playing n="+n+" > ", DIAGNOSTICS.LEVEL_2 );

          if(n==0)  //We won
          {
            // Indicate what is going on
            DIAGNOSTICS.println( _playertype+": WON > "+_opponent, DIAGNOSTICS.LEVEL_2 );
            _result= "(gameover :looser "+_opponent+" :winner "+_owner.getAID()+")";
            _gamestate= "Won";
            done();
          }
        }
        else        // We lost
        {
          // Indicate what is going on
          DIAGNOSTICS.println( _playertype+": LOST > "+_opponent, DIAGNOSTICS.LEVEL_2 );
          _result= "(gameover :winner "+_opponent+" :looser "+_owner.getAID()+")";
          _gamestate= "Lost";
          done();
        }
      }
    }
    catch(Throwable t)
    {
      DIAGNOSTICS.println("Unable to reply: "+t.getMessage(), this, DIAGNOSTICS.LEVEL_MAX);
      done();  //If can't reply, just exist - results are already set to failed
    }
  }

  /**
   * Forward the message and set the new timeout.
   */
  protected void forward(ACL a)
  {
    if(_ttask!=null)
      _ttask.stop();

    _ttask= new TimeoutTask();
    newTask(_ttask, TIMEOUT);

    super.forward(a);
  }

  public void timeoutPingPongTask_TimeoutTask(Task t)
  {
    done();
  }

  public void donePingPongTask_TimeoutTask(Task t)
  {
    // Just to disable warnings
  }

  protected void done()
  {
    if(_ttask!=null)
      _ttask.stop();
    super.done();
  }

  private String Neg(String s)
  {
    if(s.equalsIgnoreCase("ping"))
      return "pong";
    else
      return "ping";
  }

  public String GetResult()
  {
    return _result;
  }

  public String GetGameState()
  {
    return _gamestate;
  }

  public AgentID GetOpponent()
  {
    return _opponent;
  }

  private class TimeoutTask extends Task
  {
    public void stop()
    { this.done(); }
  }

}