/******************************************************************************
*   Open Source Copyright Notice and License: MessageSpy for FIPA-OS
*
* Copyright notice:
*    This file is Copyright (C) 2001 Gerstner Laboratory, Czech Technical
*    University in Prague (CTU), Karlovo namesti 13, 121 35 Prague 2,
*    Czech Republic. All rights reserved.
*
* The contents of this file are subject to the Emorphia Public License
* (the "License"); you may not use this file except in compliance with the
* License.  You may obtain a copy of the License at http://www.emorphia.com/EPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
* for the specific language governing rights and limitations under the License.
*
* The Original Code is MessageSpy for FIPA-OS (by Gerstner Laboratory).
*
* The Initial Developer of the Original Code is Gerstner Laboratory.
*
* Contributor(s):
*   Michal Dobisek, Gerstner Laboratory
*
******************************************************************************/

/**
 * Title:        PingPong agent for MessageSpy testing
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Gerstner Laboratory, FEE-CTU, Prague
 * @author Michal Dobek, michaldobisek@bigfoot.com
 * @version 1.0
 */

package bp.pingpong;

// Agent types
import fipaos.agent.task.Task;
import fipaos.agent.conversation.*;

// Ontology types
import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;

// Finally import the diagnostics class for output
import fipaos.util.DIAGNOSTICS;

/**
 * Title:        Ping Pong Agent Implementation
 * Description:  My first MAS
 * Copyright:    Copyright (c) 2001
 * Company:      none
 * @author MiDo
 * @version 1.0
 */

public class PingTask extends PingPongTask {

  /**
   * Initiator is the original message that requested the game
   */
  private Conversation _initiator= null;
  private AgentID _initiatorAID= null;
  private int _n;

  public PingTask(Conversation Initiator, AgentID Opponent, int n)
  {
    super("ping");
    _initiator= Initiator;
    if(_initiator!=null)
      _initiatorAID= _initiator.getSenderAID(_initiator.getLatestMessageIndex());
    _opponent= Opponent;
    _n= n;

/*
    // Deparse the last message
    ACL acl = conv.getACL(conv.getLatestMessageIndex());
    Constructor msg= new Constructor(acl);

    // Get the parameters
    _opponent= new AgentID(msg.GetStrParam(":opponent"));
    _n= msg.GetIntParam(":count");
*/
  }

  protected void startTask()
  {
    super.startTask();

    try
    {
      // Create an ACLMessage and forward it to our target Agent
      ACL acl = getNewConversation(null);
      acl.setPerformative( FIPACONSTANTS.REQUEST );
      acl.setSenderAID( _owner.getAID() );
      acl.setReceiverAID( _opponent );
      acl.setContentObject( "("+_playertype+" :count " + _n + ")" );
      acl.setOntology( "ping-pong" );

      // Forward via super.forward(), which binds the new Conversation with this Task
      forward( acl );

      // Indicate what is going on
      DIAGNOSTICS.println( _playertype+": GAME START > "+_opponent, DIAGNOSTICS.LEVEL_2 );

      if(_n==0)
      {
        DIAGNOSTICS.println( _playertype+": WON > "+_opponent, DIAGNOSTICS.LEVEL_2 );
        _result= "(gameover :looser "+_opponent+" :winner "+_owner.getAID()+")";
        _gamestate= "Won";
        done();
      }
    }
    catch(Throwable t)
    {
      done(); //Just exist - results are already set to failed
    }
  }

  public Conversation GetInitiator()
  {
    return _initiator;
  }

  public AgentID GetInitiatorAID()
  {
    return _initiatorAID;
  }
}